import syft as sy
import torch
import torchvision
import torchvision.transforms as transforms
from torch.utils.data import DataLoader
import numpy as np

def main():
    # 启动服务器
    data_site = sy.orchestra.launch(
        name="cifar10-server",
        reset=True,
        port=8093,
    )

    try:
        # 使用默认凭证登录
        client = data_site.login(
            email="info@openmined.org",
            password="changethis"
        )

        # 设置数据所有者账户
        OWNER_EMAIL = "owner@cifar10.research"
        OWNER_PASSWD = "cifar10_research_syft_admin"
        
        # 更新账户信息
        client.account.set_email(OWNER_EMAIL)
        client.account.set_password(OWNER_PASSWD, confirm=False)
        client.account.update(
            name="Data Owner",
            institution="CIFAR10 Research Center"
        )

        # 使用新凭证重新登录
        client = data_site.login(
            email=OWNER_EMAIL,
            password=OWNER_PASSWD
        )

        # 加载CIFAR10数据集
        transform = transforms.Compose([
            transforms.ToTensor(),
            transforms.Normalize((0.5, 0.5, 0.5), (0.5, 0.5, 0.5))
        ])

        trainset = torchvision.datasets.CIFAR10(
            root='./data',
            train=True,
            download=True,
            transform=transform
        )

        # 创建数据加载器
        trainloader = DataLoader(trainset, batch_size=4, shuffle=True)

        # 获取一些样本数据
        dataiter = iter(trainloader)
        images, labels = next(dataiter)

        # 创建模拟数据
        mock_images = images + torch.randn_like(images) * 0.1
        mock_labels = labels[torch.randperm(len(labels))]

        # 创建资产
        images_asset = sy.Asset(
            name="CIFAR10 Images",
            data=images,
            mock=mock_images,
            description="CIFAR10 image data"
        )

        labels_asset = sy.Asset(
            name="CIFAR10 Labels",
            data=labels,
            mock=mock_labels,
            description="CIFAR10 label data"
        )

        # 数据集元数据
        description = """
        CIFAR10 Dataset
        This dataset contains 60,000 32x32 color images in 10 classes.
        Classes: airplane, automobile, bird, cat, deer, dog, frog, horse, ship, truck
        """
        
        summary = "The CIFAR10 dataset is a collection of images that are commonly used to train machine learning and computer vision algorithms."
        
        citation = "Krizhevsky, A. (2009). Learning Multiple Layers of Features from Tiny Images. Technical Report TR-2009, University of Toronto."
        
        # 创建数据集
        cifar10_dataset = sy.Dataset(
            name="CIFAR10 Training Dataset",
            description=description,
            summary=summary,
            citation=citation,
            asset_list=[images_asset, labels_asset]
        )

        # 上传数据集
        client.upload_dataset(dataset=cifar10_dataset)
        print("Dataset uploaded successfully!")

        # 创建研究者账户
        researcher_account_info = client.users.create(
            email="researcher@cifar10.research",
            name="Dr. Researcher",
            password="syftrocks",
            password_verify="syftrocks",
            institution="CIFAR10 Research Institute",
            website="https://cifar10.research.data"
        )
        print(f"New User: {researcher_account_info.name} ({researcher_account_info.email}) registered as {researcher_account_info.role}")

        # 保持服务器运行
        print("Server is running. Press Enter to stop...")
        input()

    finally:
        data_site.land()

if __name__ == '__main__':
    main()